{*******************************************************}
{                                                       }
{       Methods of UMeng                                }
{                                                       }
{       CopyRight (C) 2018-2020 KngStr                  }
{                                                       }
{*******************************************************}

unit UMeng;

//https://developer.umeng.com/docs/66632/detail/66889
//delphi ֧Android 64Բ64λ soarm64-v8a

interface

uses
  System.Sysutils, System.Generics.Collections;

type
  TUMeng = class
  private
    class var FLastViewName: string;
    class function DeviceType(IsPhone: Boolean): Integer;
    class procedure Debug;
  public
    //ע: ʹѾAndroidManifest.xmlùappkeychannelֵ
    //ҲҪAppеóʼӿڣҪʹAndroidManifest.xmlúõappkeychannelֵ
    //UMConfigure.initappkeychannelΪnull
    class procedure Init(appkey, channel: string; pushSecret: string = ''; IsPhone: Boolean = True); overload;
    //ע⣺ѾAndroidManifest.xmlùappkeychannelֵԵô˰汾ʼ
    class procedure Init(pushSecret: string = ''; IsPhone: Boolean = True); overload;
    class procedure onResume;
    class procedure onPause;
    class procedure onPageStart(ViewName: string);
    class procedure onPageEnd(ViewName: string);
    class procedure onPageAutoChange(ViewName: string = ''; NeedPauseAndResume: Boolean = False);
    //ͨ¼
    class procedure onEvent(AID: string; ALabel: string = #0); overload;
    //(K-V)¼
    class procedure onEvent(AID: string; AProperties: TArray<TPair<string, string>>); overload;
    //ֵ
    class procedure onEventValue(AID: string; AProperties: TArray<TPair<string, string>>; ADuration: Integer = 0);
    //ϴηʵҳ
    class property LastViewName: string read FLastViewName;

    //ڲע¼
    class procedure onEventRegister(AUserID: string);
    //ڲ¼¼
    class procedure onEventLogin(AUserID: string);
    //ڲȡ¼
    class procedure onEventReceiveCoupon(AAmount: Integer);
    //ڲղ¼
    class procedure onEventCollect(AItem: string);
    //ڲӹﳵ¼
    class procedure onEventAddCart(AItem: string; AAmount: Integer);
    //ڲ¶¼
    class procedure onEventSubmitPayment(AUserID, AOrderId, AItem: string; AAmount: Integer);
    //ڲ¼
    class procedure onEventFinishPayment(AUserID, AOrderId, AItem: string; AAmount: Integer);
    //ڲԶ¼
    class procedure onEventCustom(AID: Integer);
  end;

implementation

uses
{$IFDEF ANDROID}
  Androidapi.JNI.Os, Androidapi.JNI.Provider,
  Androidapi.JNIBridge, Androidapi.Helpers,
  Androidapi.JNI.JavaTypes, Androidapi.JNI.App,
  Androidapi.JNI.GraphicsContentViewText,
  Androidapi.JNI.umeng.common, Androidapi.JNI.umeng.analytics,
{$ENDIF}
  FMX.Types;

{ TUMeng }

class procedure TUMeng.Debug;
{$IF DEFINED(ANDROID) && DEFINED(DEBUG)}
var
  I: Integer;
  sDeviceInfo: string;
  arrInfo: TJavaObjectArray<JString>;
{$ENDIF}
begin
{$IF DEFINED(ANDROID) && DEFINED(DEBUG)}
  (**
  * Log
  * : boolean ĬΪfalse鿴LOGΪtrue
  *)
  TJUMConfigure.JavaClass.setLogEnabled(True);
  TJMobclickAgent.JavaClass.setDebugMode(True);

  sDeviceInfo := '';
  arrInfo := TJUMConfigure.JavaClass.getTestDeviceInfo(
    {$IF CompilerVersion > 27}
    TAndroidHelper.Context
    {$ELSE}
    SharedActivityContext
    {$ENDIF}
  );

  for I := 0 to arrInfo.Length - 1 do
    sDeviceInfo := Format('%s %s', [sDeviceInfo, JStringToString(arrInfo.Items[I])]);

  Log.d('TUMeng-DeviceInfo:' + sDeviceInfo);

  //setCatchUncaughtExceptions can not be called in child process
  //TJMobclickAgent.JavaClass.setCatchUncaughtExceptions(True);
{$ENDIF}
end;

class procedure TUMeng.onEvent(AID: string; ALabel: string = #0);
begin
{$IFDEF ANDROID}
  TJMobclickAgent.JavaClass.onEvent(
    {$IF CompilerVersion > 27}
    TAndroidHelper.Context
    {$ELSE}
    SharedActivityContext
    {$ENDIF},
    StringToJString(AID),
    StringToJString(ALabel)
  );
{$ENDIF}
end;

class function TUMeng.DeviceType(IsPhone: Boolean): Integer;
begin
{$IFDEF ANDROID}
  if IsPhone then
    Result := TJUMConfigure.JavaClass.DEVICE_TYPE_PHONE
  else
    Result := TJUMConfigure.JavaClass.DEVICE_TYPE_BOX;
{$ENDIF}
end;

(**
* ʼcommon
* 1:ģΪ
* 2:豸ͣUMConfigure.DEVICE_TYPE_PHONEΪֻUMConfigure.DEVICE_TYPE_BOXΪӣĬΪֻ
* 3:Pushҵsecret
*)
class procedure TUMeng.Init(pushSecret: string; IsPhone: Boolean);
begin
{$IFDEF DEBUG}
  Debug;
{$ENDIF}

  FLastViewName := '';

{$IFDEF ANDROID}
  TJUMConfigure.TU2Fixinit(
    {$IF CompilerVersion > 27}
    TAndroidHelper.Context
    {$ELSE}
    SharedActivityContext
    {$ENDIF},
    DeviceType(IsPhone),
    StringToJString(pushSecret));
{$ENDIF}
end;

(**
* ʼcommon
* 1:ģΪ
* 2:+ AppKey
* 3:+ Channel
* 4:豸ͣUMConfigure.DEVICE_TYPE_PHONEΪֻUMConfigure.DEVICE_TYPE_BOXΪӣĬΪֻ
* 5:Pushҵsecret
*)
class procedure TUMeng.Init(appkey, channel, pushSecret: string;
  IsPhone: Boolean);
begin
{$IFDEF DEBUG}
  Debug;
{$ENDIF}

  FLastViewName := '';

{$IFDEF ANDROID}
  TJUMConfigure.TU2Fixinit(
    {$IF CompilerVersion > 27}
    TAndroidHelper.Context
    {$ELSE}
    SharedActivityContext
    {$ENDIF},
    StringToJString(appkey),
    StringToJString(channel),
    DeviceType(IsPhone),
    StringToJString(pushSecret));

  // ҳɼģʽ
  TJMobclickAgent.JavaClass.setPageCollectionMode(TJMobclickAgent_PageMode.JavaClass.AUTO);
{$ENDIF}
end;

class procedure TUMeng.onEvent(AID: string;
  AProperties: TArray<TPair<string, string>>);
begin
  onEventValue(AID, AProperties);
end;

class procedure TUMeng.onEventRegister(AUserID: string);
begin
  TUMeng.onEventValue('__register', [TPair<string, string>.Create('userid', AUserID)]);
end;

class procedure TUMeng.onEventLogin(AUserID: string);
begin
  TUMeng.onEventValue('__login', [TPair<string, string>.Create('userid', AUserID)]);
end;

class procedure TUMeng.onEventAddCart(AItem: string; AAmount: Integer);
begin
  TUMeng.onEventValue('__add_cart', [
    TPair<string, string>.Create('item', AItem),
    TPair<string, string>.Create('amount', IntToStr(AAmount))
  ]);
end;

class procedure TUMeng.onEventReceiveCoupon(AAmount: Integer);
begin
  TUMeng.onEventValue('__receive_coupon', [TPair<string, string>.Create('amount', IntToStr(AAmount))]);
end;

class procedure TUMeng.onEventSubmitPayment(AUserID, AOrderId, AItem: string;
  AAmount: Integer);
begin
  TUMeng.onEventValue('__submit_payment', [
    TPair<string, string>.Create('userid', AUserID),
    TPair<string, string>.Create('orderid', AOrderId),
    TPair<string, string>.Create('item', AItem),
    TPair<string, string>.Create('amount', IntToStr(AAmount))
  ]);
end;

class procedure TUMeng.onEventFinishPayment(AUserID, AOrderId, AItem: string;
  AAmount: Integer);
begin
  TUMeng.onEventValue('__finish_payment', [
    TPair<string, string>.Create('userid', AUserID),
    TPair<string, string>.Create('orderid', AOrderId),
    TPair<string, string>.Create('item', AItem),
    TPair<string, string>.Create('amount', IntToStr(AAmount))
  ]);
end;

class procedure TUMeng.onEventCollect(AItem: string);
begin
  TUMeng.onEventValue('__collect', [TPair<string, string>.Create('item', AItem)]);
end;

class procedure TUMeng.onEventCustom(AID: Integer);
begin
  TUMeng.onEvent('__cust_event_' + IntToStr(AID));
end;

class procedure TUMeng.onEventValue(AID: string;
  AProperties: TArray<TPair<string, string>>; ADuration: Integer);
{$IFDEF ANDROID}
var
  FJMap: JHashMap;
  I: Integer;
  LProperty: TPair<string, string>;
{$ENDIF}
begin
{$IFDEF ANDROID}
  FJMap := TJHashMap.JavaClass.init;
  for I := 0 to Length(AProperties) - 1 do
    FJMap.put(StringToJString(AProperties[I].Key), StringToJString(AProperties[I].Value));

  if ADuration > 0 then
    TJMobclickAgent.JavaClass.onEventValue(
      {$IF CompilerVersion > 27}
      TAndroidHelper.Context
      {$ELSE}
      SharedActivityContext
      {$ENDIF},
      StringToJString(AID),
      Jmap(FJMap),
      ADuration
    )
  else
    TJMobclickAgent.JavaClass.onEvent(
      {$IF CompilerVersion > 27}
      TAndroidHelper.Context
      {$ELSE}
      SharedActivityContext
      {$ENDIF},
      StringToJString(AID),
      Jmap(FJMap)
    );
{$ENDIF}
end;

class procedure TUMeng.onPageAutoChange(ViewName: string; NeedPauseAndResume: Boolean);
begin
  {$IFDEF DEBUG}
  Log.d('---onPageAutoChange:' + FLastViewName + '/' + ViewName);
  {$ENDIF}

  if FLastViewName <> '' then begin
    onPageEnd(FLastViewName);
    if NeedPauseAndResume then
      onPause;
  end;

  if ViewName <> '' then begin
    onPageStart(ViewName);
    if NeedPauseAndResume then
      onResume;
  end;

  FLastViewName := ViewName;
end;

class procedure TUMeng.onPageEnd(ViewName: string);
begin
{$IFDEF DEBUG}
  Log.d('---onPageEnd:' + ViewName);
{$ENDIF}
{$IFDEF ANDROID}
  TJMobclickAgent.JavaClass.onPageEnd(StringToJString(ViewName));
{$ENDIF}
end;

class procedure TUMeng.onPageStart(ViewName: string);
begin
{$IFDEF DEBUG}
  Log.d('---onPageStart:' + ViewName);
{$ENDIF}
{$IFDEF ANDROID}
  TJMobclickAgent.JavaClass.onPageStart(StringToJString(ViewName));
{$ENDIF}
end;

class procedure TUMeng.onPause;
begin
{$IFDEF DEBUG}
  Log.d('---onPause');
{$ENDIF}
{$IFDEF ANDROID}
  TJMobclickAgent.JavaClass.onPause(
    {$IF CompilerVersion > 27}
    TAndroidHelper.Context
    {$ELSE}
    SharedActivityContext
    {$ENDIF}
  );
{$ENDIF}
end;

class procedure TUMeng.onResume;
begin
{$IFDEF DEBUG}
  Log.d('---onResume');
{$ENDIF}
{$IFDEF ANDROID}
  TJMobclickAgent.JavaClass.onResume(
    {$IF CompilerVersion > 27}
    TAndroidHelper.Context
    {$ELSE}
    SharedActivityContext
    {$ENDIF}
  );
{$ENDIF}
end;

end.
